"""
    Cardigan - Disease Gene Prediction Method
    Copyright (C) 2018  Juan Caceres

    Cardigan is licensed under a Creative Commons Attribution-NonCommercial 4.0 
    International License.  If you did not get a copy of the license with
    the source code, you can get a copy at:
    
    https://creativecommons.org/licenses/by-nc/4.0/legalcode

"""
__author__  = "Juan Caceres"
__email__   = "Juan.CaceresSilva.2014@live.rhul.ac.uk"
__copyright__ = "Copyright (C) 2018 Juan Caceres"
__license__ = "CC BY-NC"
__version__ = "1.0"

from collections import defaultdict


class ColumnMismatch(Exception):
    pass

class ParsedLine(object):
	def __init__(self, columns, separator, strict,  *args):
		self.__mapping = defaultdict()
		if len(args) == 1:
			args = args[0].split(separator)
			
		if strict and len(columns) != len(args):
			raise ColumnMismatch("Different number of columns found in file")

		for (name, value) in zip(columns,args):
			setattr(self, name, value.strip())

class Parser(object):
	def __init__(self, filename, columns, separator = '\t', strict = False):
		self.__filename__ = filename
		self.__columns__ = columns
		self.__separator__ = separator
		self.__strict__ = strict


	def mapping(self):
		with open(self.__filename__,'r') as mapFile:
			for line in mapFile:
				if line[0]=='#':
					continue
				try:
					yield ParsedLine(self.__columns__,self.__separator__, self.__strict__, line)
				except TypeError, ex:
					print "Parse Error. Please check the file"
					exit()	
				except ColumnMismatch as e:
					continue

	def __iter__(self):
		return self.mapping()